"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetupOpenVpn = void 0;
const chalk_1 = __importDefault(require("chalk"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const fs_1 = __importDefault(require("fs"));
const package_json_1 = __importDefault(require("../package.json"));
const shelljs_1 = __importDefault(require("shelljs"));
class SetupOpenVpn {
    constructor(options) {
        this.options = options;
        this.s3Client = new aws_sdk_1.default.S3({
            region: options.region
        });
        this.runSetup();
    }
    runSetup() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Running openvpn unattended setup ${package_json_1.default.version}`));
            this.setupOpenVpn();
            this.setupSsl();
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Something failed during openvpn setup`), e);
            return;
        }
    }
    setupOpenVpn() {
        const { userName, userPassword, ip } = this.options;
        if (typeof ip !== 'undefined') {
            console.log(chalk_1.default.bgGreenBright('Setting upstream dns settings'));
            try {
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.client.routing.reroute_dns --value custom ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.server.dhcp_option.dns.0 --value ${ip} ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.server.routing.gateway_access --value true ConfigPut`);
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright('An error occured while setting upstream dns settings'), e);
                throw e;
            }
        }
        try {
            console.log(chalk_1.default.bgGreenBright('Creating default client user for OpenVPN'));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.client.routing.reroute_gw --value true ConfigPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName} --key type --value user_connect UserPropPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName} --key prop_autologin --value true UserPropPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName} --key new_pass --value ${userPassword} SetLocalPassword`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while creating default client user'), e);
            throw e;
        }
    }
    setupSsl() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(chalk_1.default.bgGreenBright(`Starting process to append an SSL certificate to OpenVPN Access Server Web UI`));
            const { domainName, email, bucket } = this.options;
            this.installCertbot();
            const hasExistingCert = yield this.hasExistingCert();
            if (!hasExistingCert) {
                console.log(chalk_1.default.yellowBright(`No existing certificate to reuse, creating new cert`));
                try {
                    shelljs_1.default.exec([
                        'sudo',
                        'certbot',
                        'certonly',
                        '--standalone',
                        '--server',
                        'https://acme-staging-v02.api.letsencrypt.org/directory',
                        '--non-interactive',
                        '--agree-tos',
                        '--email',
                        email,
                        '--domains',
                        domainName
                    ].join(' '));
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright('An error occured while generating a cert'), e);
                    throw e;
                }
                try {
                    shelljs_1.default.exec(`sudo chmod -R 755 /etc/letsencrypt`);
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright('An error while chaging directory permissions'), e);
                    throw e;
                }
                try {
                    const certPem = fs_1.default.readFileSync(`/etc/letsencrypt/live/${domainName}/cert.pem`, {
                        encoding: 'utf8'
                    });
                    const privKeyPem = fs_1.default.readFileSync(`/etc/letsencrypt/live/${domainName}/privkey.pem`, {
                        encoding: 'utf8'
                    });
                    const chainPem = fs_1.default.readFileSync(`/etc/letsencrypt/live/${domainName}/chain.pem`, {
                        encoding: 'utf8'
                    });
                    const fullChainPem = fs_1.default.readFileSync(`/etc/letsencrypt/live/${domainName}/fullchain.pem`, {
                        encoding: 'utf8'
                    });
                    yield this.s3Client
                        .putObject({
                        Bucket: bucket,
                        Key: `letsencrypt/${domainName}/cert.pem`,
                        Body: certPem
                    })
                        .promise();
                    yield this.s3Client
                        .putObject({
                        Bucket: bucket,
                        Key: `letsencrypt/${domainName}/privkey.pem`,
                        Body: privKeyPem
                    })
                        .promise();
                    yield this.s3Client
                        .putObject({
                        Bucket: bucket,
                        Key: `letsencrypt/${domainName}/chain.pem`,
                        Body: chainPem
                    })
                        .promise();
                    yield this.s3Client
                        .putObject({
                        Bucket: bucket,
                        Key: `letsencrypt/${domainName}/fullchain.pem`,
                        Body: fullChainPem
                    })
                        .promise();
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright(`An error occured when copying certs to S3 bucket ${bucket}`), e);
                    throw e;
                }
            }
            else {
                console.log(chalk_1.default.bgGreenBright(`Existing cert found, reusing it...`));
                try {
                    shelljs_1.default.exec([
                        'sudo',
                        'mkdir',
                        '-p',
                        `/etc/letsencrypt/live/${domainName}`
                    ].join(' '));
                    shelljs_1.default.exec(['sudo', 'chmod', '-R', '777', `/etc/letsencrypt`].join(' '));
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright(`An error occured while creating relevant directories to store certificate`), e);
                    throw e;
                }
                try {
                    this.writeToFile(yield this.getS3Object(bucket, `letsencrypt/${domainName}/cert.pem`), `/etc/letsencrypt/live/${domainName}/cert.pem`);
                    this.writeToFile(yield this.getS3Object(bucket, `letsencrypt/${domainName}/privkey.pem`), `/etc/letsencrypt/live/${domainName}/privkey.pem`);
                    this.writeToFile(yield this.getS3Object(bucket, `letsencrypt/${domainName}/chain.pem`), `/etc/letsencrypt/live/${domainName}/chain.pem`);
                    this.writeToFile(yield this.getS3Object(bucket, `letsencrypt/${domainName}/fullchain.pem`), `/etc/letsencrypt/live/${domainName}/fullchain.pem`);
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright(`An error occured while getting certificate artifacts from ${bucket}`), e);
                    throw e;
                }
            }
            this.applyCert();
            this.startOpenVpn();
        });
    }
    getS3Object(Bucket, Key) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.s3Client
                    .getObject({
                    Bucket,
                    Key
                })
                    .promise();
                if (typeof response.Body === 'undefined') {
                    throw new Error(`Body of ${Key} is undefined`);
                }
                return (_a = response.Body) === null || _a === void 0 ? void 0 : _a.toString('utf-8');
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright(`Failed to get ${Key} from ${Bucket}`));
                throw e;
            }
        });
    }
    writeToFile(content, path) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                fs_1.default.writeFileSync(path, content);
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright(`Failed to write ${path}`));
                throw e;
            }
        });
    }
    hasExistingCert() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log(chalk_1.default.blueBright(`Checking for existing SSL certificate`));
                const results = yield Promise.all([
                    yield this.s3Client
                        .headObject({
                        Bucket: this.options.bucket,
                        Key: `letsencrypt/${this.options.domainName}/cert.pem`
                    })
                        .promise(),
                    yield this.s3Client
                        .headObject({
                        Bucket: this.options.bucket,
                        Key: `letsencrypt/${this.options.domainName}/privkey.pem`
                    })
                        .promise(),
                    yield this.s3Client
                        .headObject({
                        Bucket: this.options.bucket,
                        Key: `letsencrypt/${this.options.domainName}/chain.pem`
                    })
                        .promise(),
                    yield this.s3Client
                        .headObject({
                        Bucket: this.options.bucket,
                        Key: `letsencrypt/${this.options.domainName}/fullchain.pem`
                    })
                        .promise()
                ]);
                return true;
            }
            catch (e) {
                console.log(chalk_1.default.yellowBright(`Required certificate artifacts could not be found in ${this.options.bucket}`), e);
                return false;
            }
        });
    }
    installCertbot() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Installing certbot`));
            shelljs_1.default.exec(`sudo apt-get -y install software-properties-common`);
            shelljs_1.default.exec(`sudo add-apt-repository ppa:certbot/certbot -y`);
            shelljs_1.default.exec(`sudo apt-get -y update`);
            shelljs_1.default.exec(`sudo apt-get -y install certbot`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while installing certbot'), e);
            throw e;
        }
    }
    applyCert() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Delete existing cert references in OpenVPN DB`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.ca_bundle`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.priv_key`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.cert`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while deleting certificate references in OpenVPN DB'), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Apply symbolic links to SSL certificates`));
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/cert.pem /usr/local/openvpn_as/etc/web-ssl/server.crt`);
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/privkey.pem /usr/local/openvpn_as/etc/web-ssl/server.key`);
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/chain.pem /usr/local/openvpn_as/etc/web-ssl/ca.crt`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured symlinking to certificate files'), e);
            throw e;
        }
    }
    stopOpenVpn() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Stopping openvpn`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli stop`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while stopping openvpn'), e);
            return;
        }
    }
    startOpenVpn() {
        try {
            console.log(chalk_1.default.blueBright(`Starting openvpn`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli start`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while starting openvpn'), e);
            return;
        }
    }
}
exports.SetupOpenVpn = SetupOpenVpn;
//# sourceMappingURL=openvpn-unattended-install.js.map